<?php
require_once('wp-plugin-retina.php');

if (!class_exists('wp_retina')) {
    class wp_retina extends WPPlugin_retina {
        // member variables
        private $saved_error;
		private $RetinaPost;
        
        // php 4 constructor
        function wp_retina($options_name) {
            $args = func_get_args();
            call_user_func_array(array(&$this, "__construct"), $args);
        }
        
        // php 5 constructor
        function __construct($options_name) {
            parent::__construct($options_name);
			
			// create RetinaPosta object 
			require_once($this->path_to_plugin_directory() . '/Retina_Library.php');
			$this->RetinaPost = new RetinaPost();
            
            $this->register_default_options();
            
            // register the hooks
            $this->register_actions();
            $this->register_filters();
        }
        
        function register_actions() {
            
			// load the plugin's textdomain for localization
            /*dd retina doesn't have languages yet*/ // add_action('init', array(&$this, 'load_textdomain'));
        
             // options  // run when the plugin is activated
            register_activation_hook(WPPlugin_retina::path_to_plugin_directory() . '/retina-post.php', array(&$this, 'register_default_options')); 
            add_action('admin_init', array(&$this, 'register_settings_group'));
			
			// only register the hooks if the user wants retina on the comments page
            if ($this->options['show_in_comments']) {
                add_action('comment_form', array(&$this, 'show_retina_in_comments'));
                //DD: it doesn't do what it says add_action('wp_footer', array(&$this, 'save_comment_script')); // preserve the comment that was entered

                // retina comment processing (look into doing all of this with AJAX, optionally)
                add_action('wp_head', array(&$this, 'saved_comment'), 0);
                add_action('preprocess_comment', array(&$this, 'check_comment'), 0);
                add_action('comment_post_redirect', array(&$this, 'relative_redirect'), 0, 2);
            }

            // only register the hooks if the user wants retina on the registration page
            if ($this->options['show_in_registration']) {
                // retina form display
                if ($this->is_multi_blog())
                    add_action('signup_extra_fields', array(&$this, 'show_retina_in_registration'));
                else
                    add_action('register_form', array(&$this, 'show_retina_in_registration'));
            }

            // administration (menus, pages, notifications, etc.)
            add_filter("plugin_action_links", array(&$this, 'show_settings_link'), 10, 2);

            add_action('admin_menu', array(&$this, 'add_settings_page'));
            
            // admin notices
            add_action('admin_notices', array(&$this, 'show_admin_notice'));
        }
        
        function register_filters() {
            // only register the hooks if the user wants retina on the registration page
            if ($this->options['show_in_registration']) {
                // retina validation
                if ($this->is_multi_blog())
                    add_filter('wpmu_validate_user_signup', array(&$this, 'validate_retina_response_wpmu'));
                else
                    add_filter('registration_errors', array(&$this, 'validate_retina_response'));
            }
        }
        
        // set the default options
        function register_default_options() {
            if ($this->options)
               return;
           
            $option_defaults = array();
           
			$old_options = WPPlugin_retina::retrieve_options($this->options_name);
           
		     
            if ($old_options) {
               $option_defaults['public_key'] = $old_options['public_key']; // the public key for retina
               $option_defaults['private_key'] = $old_options['private_key'];; // the private key for retina
			   
			   $option_defaults['show_in_comments'] = $old_options['show_in_comments'];; // whether or not to show retina on the comment post
			   $option_defaults['bypass_for_registered_users'] = $old_options['bypass_for_registered_users'];; // whether to skip retinas for registered users
               $option_defaults['minimum_bypass_level'] = $old_options['minimum_bypass_level'];; // who doesn't have to do the retina (should be a valid WordPress capability slug)
               
			   $option_defaults['feed_URL_c'] = $old_options['feed_URL_c']; 	// empty feed
			   $option_defaults['feed_limit_c'] = $old_options['feed_limit_c']; 	// show last ten feeds
			   $option_defaults['custom_title_c'] = $old_options['custom_title_c'];// empty custom title
			   $option_defaults['custom_description_c'] = $old_options['custom_description_c']; // empty custom description
			   $option_defaults['custom_link_c'] = $old_options['custom_link_c']; 	// empty custom link
			   			   
               $option_defaults['show_in_registration'] = $old_options['show_in_registration']; // whether or not to show retina on the registration page
			   $option_defaults['feed_URL_r'] = $old_options['feed_URL_r']; 	// empty feed
			   $option_defaults['feed_limit_r'] = $old_options['feed_limit_r']; 	// show last ten feeds
			   $option_defaults['custom_title_r'] = $old_options['custom_title_r']; // empty custom title
			   $option_defaults['custom_description_r'] = $old_options['custom_description_r']; // empty custom description
			   $option_defaults['custom_link_r'] = $old_options['custom_link_r']; 	// empty custom link
                                  
               $option_defaults['xhtml_compliance'] = $old_options['xhtml_compliance']; // whether or not to be XHTML 1.0 Strict compliant
               $option_defaults['comments_tab_index'] = $old_options['comments_tab_index']; // the default tabindex for retina
               $option_defaults['registration_tab_index'] = $old_options['registration_tab_index']; // the default tabindex for retina

               $option_defaults['call_to_action'] = $old_options['call_to_action'];  // tell user what to do to complete he CAPTCHA
               $option_defaults['incorrect_response_error'] = $old_options['incorrect_response_error']; // message for incorrect CAPTCHA response
            }
           
            else {
               $option_defaults['public_key'] = ''; // the public key for retina
               $option_defaults['private_key'] = ''; // the private key for retina

               $option_defaults['show_in_comments'] = 1; // whether or not to show retina on the comment post
			   $option_defaults['bypass_for_registered_users'] = 0; // whether to skip retinas for registered users
               $option_defaults['minimum_bypass_level'] = 'read'; // who doesn't have to do the retina (should be a valid WordPress capability slug)
               
			   $option_defaults['feed_URL_c'] = ''; 	// empty feed
			   $option_defaults['feed_limit_c'] = 10; 	// show last ten feeds
			   $option_defaults['custom_title_c'] = ''; // empty custom title
			   $option_defaults['custom_description_c'] = ''; // empty custom description
			   $option_defaults['custom_link_c'] = ''; 	// empty custom link
			   			   
               $option_defaults['show_in_registration'] = 1; // whether or not to show retina on the registration page
			   $option_defaults['feed_URL_r'] = ''; 	// empty feed
			   $option_defaults['feed_limit_r'] = 10; 	// show last ten feeds
			   $option_defaults['custom_title_r'] = ''; // empty custom title
			   $option_defaults['custom_description_r'] = ''; // empty custom description
			   $option_defaults['custom_link_r'] = ''; 	// empty custom link
                                  
               $option_defaults['xhtml_compliance'] = 0; // whether or not to be XHTML 1.0 Strict compliant
               $option_defaults['comments_tab_index'] = 5; // the default tabindex for retina
               $option_defaults['registration_tab_index'] = 30; // the default tabindex for retina

               $option_defaults['call_to_action'] = 'Type the colored letters';  // tell user what to do to complete he CAPTCHA
               $option_defaults['incorrect_response_error'] = 'Incorrect response.'; // message for incorrect CAPTCHA response
            }
			            
            // add the option based on what environment we're in
            WPPlugin_retina::add_options($this->options_name, $option_defaults);            
        }
                        
        // register the settings
        function register_settings_group() {
            register_setting("retinapost_options_group", 'retinapost_options', array(&$this, 'validate_options'));
        }
        
            
        function retina_enabled() {
            return ($this->options['show_in_comments'] || $this->options['show_in_registration']);
        }
        
        function keys_missing() {
            return (empty($this->options['public_key']) || empty($this->options['private_key']));
        }
		function feed_or_custom_text_missing($were){
			return (empty($this->options['feed_URL_'.$were]) && (empty($this->options['custom_title_'.$were])||empty($this->options['custom_description_'.$were])));
		}
		
        
        function create_error_notice($message, $anchor = '') {
            $options_url = admin_url('options-general.php?page=retina-post/wp-retina.php') . $anchor;
            $error_message = sprintf(__($message . ' <a href="%s" title="Retina Settings">Go to settings</a>', 'retinapost'), $options_url);
            
            echo '<div class="error"><p><strong>' . $error_message . '</strong></p></div>';
        }
        
        function show_admin_notice() {
            if ($this->retina_enabled() && $this->keys_missing()) {
                $this->create_error_notice('Retina API Keys are empty.');
            }
            if ($this->options['show_in_comments'] && $this->feed_or_custom_text_missing('c')) {	// $were = 'c' from comments
                $this->create_error_notice('You need to provide a Feed or a Title and Description in Retina Comment Options');
            }
			if ($this->options['show_in_registration'] && $this->feed_or_custom_text_missing('r')) { // $were = 'r' from registration
                $this->create_error_notice('You need to provide a Feed or a Title and Description in Retina Registration Options');
            }
        }
        
        function validate_dropdown($array, $key, $value) {
            // make sure that the capability that was supplied is a valid capability from the drop-down list
            if (in_array($value, $array))
                return $value;
            else // if not, load the old value
                return $this->options[$key];
        }
        
        function validate_options($input) {
            // todo: make sure that 'incorrect_response_error' is not empty, prevent from being empty in the validation phase
            
            // trim the spaces out of the key, as they are usually present when copied and pasted
            $validated['public_key'] = trim($input['public_key']);
            $validated['private_key'] = trim($input['private_key']);
						
            $validated['show_in_comments'] = (isset($input['show_in_comments']) && intval($input['show_in_comments']) == 1) ? 1: 0;
            $validated['bypass_for_registered_users'] = (isset($input['bypass_for_registered_users']) && intval($input['bypass_for_registered_users']) == 1) ? 1: 0;
            
            $capabilities = array ('read', 'edit_posts', 'publish_posts', 'moderate_comments', 'activate_plugins');
                                 
            $validated['minimum_bypass_level'] = $this->validate_dropdown($capabilities, 'minimum_bypass_level', $input['minimum_bypass_level']);
			                        
			$validated['feed_URL_c'] = 		trim($input['feed_URL_c']);	
			$validated['feed_limit_c'] = 	(intval($input['feed_limit_c'])>0&&intval($input['feed_limit_c'])<20) ? intval($input["feed_limit_c"]) : 10;
			$validated['custom_title_c'] = 	trim($input['custom_title_c']);	
			$validated['custom_description_c'] = trim($input['custom_description_c']);	
			$validated['custom_link_c'] = 	trim($input['custom_link_c']);	

            $validated['comments_tab_index'] = 	(intval($input['comments_tab_index'])>0&&intval($input['comments_tab_index'])<100) ? intval($input["comments_tab_index"]) : 5;
                        
            $validated['show_in_registration'] = (isset($input['show_in_registration']) && intval($input['show_in_registration']) == 1) ? 1: 0;
            			
			$validated['feed_URL_r'] = 		trim($input['feed_URL_r']);	
			$validated['feed_limit_r'] = 	(intval($input['feed_limit_r'])>0&&intval($input['feed_limit_r'])<20) ? intval($input["feed_limit_r"]) : 10;
			$validated['custom_title_r'] = 	trim($input['custom_title_r']);	
			$validated['custom_description_r'] = trim($input['custom_description_r']);	
			$validated['custom_link_r'] = 	trim($input['custom_link_r']);	
			
			$validated['registration_tab_index'] = (intval($input['registration_tab_index'])>0&&intval($input['registration_tab_index'])<100) ? intval($input["comments_tab_index"]) : 30; 
		   
            $validated['xhtml_compliance'] = (isset($input['xhtml_compliance']) && intval($input['xhtml_compliance']) == 1) ? 1: 0;
            
            $validated['call_to_action'] = $input['call_to_action'];
            $validated['incorrect_response_error'] = $input['incorrect_response_error'];
            
            return $validated;
        }
        
        
        function show_retina_in_registration($errors) {
            // if it's for wordpress mu, show the errors
            if ($this->is_multi_blog()) {
                $error = $errors->get_error_message('captcha');
                echo '<label for="verification">Verification:</label>';
                echo ($error ? '<p class="error">'.$error.'</p>' : '');
                echo $this->get_retina_html($this->options['call_to_action'],'r');
            }            
            // for regular wordpress
            else {
                echo $this->get_retina_html($this->options['call_to_action'],'r');
            }
        }
        
        function validate_retina_response($errors) {
            // empty so throw the empty response error
            if (empty($_POST['Retina_response']) || trim($_POST['Retina_response'])=='') {
                $errors->add('blank_captcha', $this->options['incorrect_response_error']);
                return $errors;
            }
     
            if ($this->RetinaPost->check_answer())
                $errors->add('captcha_wrong', $this->options['incorrect_response_error']);

			return $errors;
        }
        
        function validate_retina_response_wpmu($result) {
            // must make a check here, otherwise the wp-admin/user-new.php script will keep trying to call
            // this function despite not having called do_action('signup_extra_fields'), so the retina
            // field was never shown. this way it won't validate if it's called in the admin interface
            
            if (!$this->is_authority()) {
                // no text entered
                if (empty($_POST['Retina_response']) || trim($_POST['Retina_response'])=='') {
                    $result['errors']->add('blank_captcha', $this->options['incorrect_response_error']);
                    return $result;
                }
                    
                // todo: why echo the error here? wpmu specific?
                if ($this->RetinaPost->check_answer()){
                    $result['errors']->add('captcha_wrong', $this->options['incorrect_response_error']);
                    //echo '<div class="error">' . $this->options['incorrect_response_error'] . '</div>';
                }                    
                return $result;
            }
        }
        
        // utility methods
        function hash_comment($id) {
            define ("retina_WP_HASH_SALT", "b7e0638d85f5d7f3694f68e944136d62");
            
            if (function_exists('wp_hash'))
                return wp_hash(retina_WP_HASH_SALT . $id);
            else
                return md5(retina_WP_HASH_SALT . $this->options['private_key'] . $id);
        }
        
        function get_retina_html($retina_message, $were, $use_ssl=false) {
            //trigger_error(the_title(), E_USER_ERROR);
			//echo get_the_title();
			
			$this->RetinaPost->publicKey = $this->options['public_key'];
			$this->RetinaPost->message   = $retina_message;
							
            if (!empty($this->options['feed_URL_'.$were]))
				return $this->RetinaPost->get_from_feed($this->options['feed_URL_'.$were], get_the_title(), $this->options['feed_limit_'.$were], $this->options['xhtml_compliance']);
			else 
				return $this->RetinaPost->get_from_text($this->options['custom_title_'.$were], $this->options['custom_description_'.$were],  $this->options['custom_link_'.$were], $this->options['xhtml_compliance']);
			 //$this->options['xhtml_compliance']
        }
        
        function show_retina_in_comments() {
            global $user_ID;

            // set the minimum capability needed to skip the captcha if there is one
            if (isset($this->options['bypass_for_registered_users']) && $this->options['bypass_for_registered_users'] && $this->options['minimum_bypass_level'])
                $needed_capability = $this->options['minimum_bypass_level'];

            // skip the retina display if the minimum capability is met
            if ((isset($needed_capability) && $needed_capability && current_user_can($needed_capability)) || !$this->options['show_in_comments'])
                return;

            else {
                // SHOW ERROR MESSAGE if there is an error
                if (!empty($_GET['rerror']))
                    echo '<p class="none" style="color:#ff3333;">' . $this->options['incorrect_response_error'] . "</p>";

                // todo: replace this with jquery: http://digwp.com/2009/06/including-jquery-in-wordpress-the-right-way/
                // todo: use math to increment+1 the submit button based on what the tab_index option is
                if ($this->options['xhtml_compliance']) {
                    $reorder_submit_button = <<<COMMENT_FORM
                        <div id="retina-submit-btn-area">&nbsp;</div>
COMMENT_FORM;
                }

                else {
                    $reorder_submit_button = <<<COMMENT_FORM
                        <div id="retina-submit-btn-area">&nbsp;</div>
                        <noscript>
                         <style type='text/css'>#submit {display:none;}</style>
                         <input name="submit" type="submit" id="submit-alt" tabindex="6" value="Submit Comment"/> 
                        </noscript>
COMMENT_FORM;
                }
				
				$reorder_submit_button.= <<<JS
                <script type="text/javascript">
                var sub = document.getElementById('submit');
                document.getElementById('retina-submit-btn-area').appendChild (sub);
                document.getElementById('submit').tabIndex = 6;
                if ( typeof _retina_wordpress_savedcomment != 'undefined') {
                        document.getElementById('comment').value = _retina_wordpress_savedcomment;
                }
                document.getElementById('retina_table').style.direction = 'ltr';
                </script>
JS;
				
                echo $this->get_retina_html($this->options['call_to_action'], 'c') . $reorder_submit_button;
           }
        }
          
        function check_comment($comment_data) {
            global $user_ID;
            
            if ($this->options['bypass_for_registered_users'] && $this->options['minimum_bypass_level'])
                $needed_capability = $this->options['minimum_bypass_level'];
            
            if ((isset($needed_capability) && current_user_can($needed_capability)) || !$this->options['show_in_comments'])
                return $comment_data;
            
			// do not check trackbacks/pingbacks
			if ($comment_data['comment_type'] == '') {
					
				$this->RetinaPost->privateKey = $this->options['private_key'];				
				if ($this->RetinaPost->check_answer())
					return $comment_data;
					
				else {
					$this->saved_error = $this->RetinaPost->get_error();
										
					// http://codex.wordpress.org/Plugin_API/Filter_Reference#Database_Writes_2
					add_filter('pre_comment_approved', create_function('$a', 'return \'spam\';'));
					return $comment_data;
				}
			}		     
            return $comment_data;
        }
        
        function relative_redirect($location, $comment) {
            if ($this->saved_error != '') {
                // replace #comment- at the end of $location with #commentform
                
                $location = substr($location, 0, strpos($location, '#')) .
                    ((strpos($location, "?") === false) ? "?" : "&") .
                    'rcommentid=' . urlencode( stripslashes($comment->comment_ID)) .
                    '&rerror=' . urlencode( stripslashes($this->saved_error)) .
                    '&rchash=' . urlencode( stripslashes($this->hash_comment($comment->comment_ID))) .
                    '#commentform';
            }            
            return $location;
        }
        
        function saved_comment() {
            if (!is_single() && !is_page())
                return;
            if (empty($_REQUEST['rcommentid']) || empty($_REQUEST['rchash']))
               return;
			
            $comment_id = $_REQUEST['rcommentid'];
            $comment_hash = $_REQUEST['rchash'];
              
            if ($comment_hash == $this->hash_comment($comment_id)) {
               $comment = get_comment($comment_id);

               // todo: removed double quote from list of 'dangerous characters'
               $com = preg_replace('/([\\/\(\)\+\;\'])/e','\'%\'.dechex(ord(\'$1\'))', $comment->comment_content);
                
               $com = preg_replace('/\\r\\n/m', '\\\n', $com);
                
               echo "
                <script type='text/javascript'>
                var _retina_wordpress_savedcomment =  '" . $com  ."';
                _retina_wordpress_savedcomment = unescape(_retina_wordpress_savedcomment);
                </script>
                ";

                wp_delete_comment($comment->comment_ID);
            }
        }
        
        // todo: is this still needed?
        // this is used for the api keys url in the administration interface
        function blog_domain() {
            $uri = parse_url(get_option('siteurl'));
            return $uri['host'];
        }
        
        // add a settings link to the plugin in the plugin list
        function show_settings_link($links, $file){
			if (basename($file) == 'retina-post.php') {
               $settings_title = __('Settings for RetinaPost', 'retinapost');
               $settings = __('Settings', 'retinapost');
               $settings_link = '<a href="options-general.php?page=retina-post/wp-retina.php" title="' . $settings_title . '">' . $settings . '</a>';
               array_unshift($links, $settings_link);
            }
            
            return $links;
        }
        
        // add the settings page
        function add_settings_page() {
            // add the options page
            if ($this->environment == Environment::WordPressMU && $this->is_authority())
                add_submenu_page('wpmu-admin.php', 'Retina Post', 'Retina Post', 'manage_options', __FILE__, array(&$this, 'show_settings_page'));

            if ($this->environment == Environment::WordPressMS && $this->is_authority())
                add_submenu_page('ms-admin.php', 'Retina Post', 'Retina Post', 'manage_options', __FILE__, array(&$this, 'show_settings_page'));
            
            add_options_page('Retina Post', 'Retina Post', 'manage_options', __FILE__, array(&$this, 'show_settings_page'));
        }
        
        // store the xhtml in a separate file and use include on it
        function show_settings_page() {
            include("settings.php");
        }
        
        function build_dropdown($name, $keyvalue, $checked_value) {
            echo '<select name="' . $name . '" id="' . $name . '">' . "\n";
            
            foreach ($keyvalue as $key => $value) {
                $checked = ($value == $checked_value) ? ' selected="selected" ' : '';
                
                echo '\t <option value="' . $value . '"' . $checked . ">$key</option> \n";
                $checked = NULL;
            }
            
            echo "</select> \n";
        }
        
        function capabilities_dropdown() {
            // define choices: Display text => permission slug
            $capabilities = array (
                __('all registered users', 'retinapost') => 'read',
                __('edit posts', 'retinapost') => 'edit_posts',
                __('publish posts', 'retinapost') => 'publish_posts',
                __('moderate comments', 'retinapost') => 'moderate_comments',
                __('activate plugins', 'retinapost') => 'activate_plugins'
            );
            
            $this->build_dropdown('retinapost_options[minimum_bypass_level]', $capabilities, $this->options['minimum_bypass_level']);
        } 

    } // end class declaration
} // end of class exists clause

?>
